#pragma semicolon 1

#include <amxmodx>
#include <fakemeta>
#include <reapi>

enum _:CvarNames {
	BLOCK_SPRAY,
	RETRY_TERR,
	BLOCK_KILL
}

new pNextTerrorist,
	Trie:g_tRemoveEntities,
	g_iForwardSpawn,
	bool:g_blCvar[CvarNames];

public plugin_init() {
	register_plugin("ReDeathrun", "0.0.1", "PurposeLess");

	register_dictionary("redeathrun.txt");

	@AmxmodxHooks();
	@ReapiHooks();
	@FakemetaHooks();
}

public plugin_precache() {
	new const szRemoveEntities[][] = {
		"func_bomb_target", "func_escapezone", "func_hostage_rescue", "func_vip_safetyzone", "info_vip_start",
        "hostage_entity", "info_bomb_target", "info_hostage_rescue", "monster_scientist"
	};

	g_tRemoveEntities = TrieCreate();

	for(new i = 0; i < sizeof(szRemoveEntities); i++) {
		TrieSetCell(g_tRemoveEntities, szRemoveEntities[i], i);
	}

	g_iForwardSpawn = register_forward(FM_Spawn, "@FM_Spawn_Pre", ._post = false);
}

public plugin_cfg() {
	bind_pcvar_num(create_cvar("rd_block_spray", "1"), g_blCvar[BLOCK_SPRAY]);
	bind_pcvar_num(create_cvar("rd_retry_terrorist", "1"), g_blCvar[RETRY_TERR]);
	bind_pcvar_num(create_cvar("rd_block_kill", "1"), g_blCvar[BLOCK_KILL]);

	new szFileName[43];
	get_localinfo("amxx_configsdir", szFileName, charsmax(szFileName));
	add(szFileName, charsmax(szFileName), "/redeathrun_core.cfg");

	if(file_exists(szFileName)) {
		server_cmd("exec %s", szFileName);
		server_exec();
	}
}

public client_putinserver(pPlayer) {
	if(get_member_game(m_bGameStarted)) {
		return;
	}

	new iPlayers[MAX_PLAYERS], iNum;
	get_players(iPlayers, iNum);

	if(iNum >= 2) {
		set_member_game(m_bGameStarted, true);
		set_pcvar_num(get_cvar_pointer("sv_restart"), 2);
	}
}

/* Thanks fl0wer */
public client_disconnected(pPlayer) {
	if(!get_member_game(m_bGameStarted)) {
		return;
	}

	new iPlayersArray[MAX_PLAYERS], iNum;

	for(new id = 1; id <= MaxClients; id++) {
		if(id == pPlayer || !is_user_connected(id)) {
			continue;
		}

		iPlayersArray[++iNum] = id;
	}

	if(iNum < 2) {
		set_member_game(m_bGameStarted, false);
		return;
	}

	if(g_blCvar[RETRY_TERR] && pNextTerrorist == pPlayer) {
		pNextTerrorist = iPlayersArray[random_num(0, iNum - 1)];
		rg_set_user_team(pNextTerrorist, TEAM_TERRORIST);
		rg_round_respawn(pNextTerrorist);
		client_print_color(0, 0, "%L", LANG_PLAYER, "REDEATHRUN_LEFT_GAME", pPlayer, pNextTerrorist);
	}
}

/* *amxmodx* */
@AmxmodxHooks() {
	register_clcmd("jointeam", "@clcmd_jointeam");
	register_clcmd("chooseteam", "@clcmd_jointeam");
}

@clcmd_jointeam(const pPlayer) {
	return PLUGIN_HANDLED;
}

/* *reapi* */
@ReapiHooks() {
	RegisterHookChain(RG_CSGameRules_RestartRound, "@CSGameRules_RestartRound_Pre", .post = false);
	RegisterHookChain(RG_CBasePlayer_ImpulseCommands, "@CBasePlayer_ImpulseCommands_Pre", .post = false);
	RegisterHookChain(RG_RoundEnd, "@RoundEnd_Post", .post = true);
	RegisterHookChain(RG_CBasePlayer_Killed, "@CBasePlayer_Killed_Post", .post = true);
	RegisterHookChain(RG_CBasePlayer_Spawn, "@CBasePlayer_Spawn_Post", .post = true);
}

@CSGameRules_RestartRound_Pre() {
	if(!get_member_game(m_bGameStarted)) {
		return;
	}

	if(!pNextTerrorist) {
		new iPlayers[32], iNum;
		get_players(iPlayers, iNum);
		pNextTerrorist = iPlayers[random(iNum)];

		client_print_color(0, 0, "%L", LANG_PLAYER, "REDEATHRUN_NEXT_TERRORIST", pNextTerrorist);
	}

	for(new pPlayer = 1; pPlayer <= MaxClients; pPlayer++) {
		if(!is_user_alive(pPlayer) || pPlayer == pNextTerrorist) {
			continue;
		}

		rg_set_user_team(pPlayer, TEAM_CT);
	}
	rg_set_user_team(pNextTerrorist, TEAM_TERRORIST);
}

@CBasePlayer_ImpulseCommands_Pre(const pPlayer) {
	if(!g_blCvar[BLOCK_SPRAY]) {
		return HC_CONTINUE;
	}

	static iImpulse;
	iImpulse = get_entvar(pPlayer, var_impulse);

	if(iImpulse == 201) {
		set_entvar(pPlayer, var_impulse, 0);
		return HC_SUPERCEDE;
	}
	return HC_CONTINUE;
}

@RoundEnd_Post() {
	if(!get_member_game(m_bGameStarted)) {
		return;
	}

	new iPlayers[32], iNum;
	get_players(iPlayers, iNum);
	pNextTerrorist = iPlayers[random(iNum)];

	client_print_color(0, 0, "%L", LANG_PLAYER, "REDEATHRUN_NEXT_TERRORIST", pNextTerrorist);
}

@CBasePlayer_Killed_Post(const pVictim, pAttacker, iGib) {
	if(get_member(pVictim, m_iTeam) == TEAM_TERRORIST) {
		rg_set_user_team(pVictim, TEAM_CT);
	}
}

@CBasePlayer_Spawn_Post(const pPlayer) {
	if(!is_user_alive(pPlayer)) {
		return;
	}

	rg_remove_all_items(pPlayer);
	rg_give_item(pPlayer, "weapon_knife");
}

/* *fakemeta* */
@FakemetaHooks() {
	register_forward(FM_ClientKill, "@ClientKill_Pre", ._post = false);
	unregister_forward(FM_Spawn, g_iForwardSpawn, .post = false);
	TrieDestroy(g_tRemoveEntities);
}

@ClientKill_Pre(const pPlayer) {
	return g_blCvar[BLOCK_KILL] ? FMRES_SUPERCEDE : FMRES_IGNORED;
}

@FM_Spawn_Pre(iEntity) {
	if(is_nullent(iEntity)) {
		return FMRES_IGNORED;
	}

	static szClassname[32];
	get_entvar(iEntity, var_classname, szClassname, charsmax(szClassname));

	if(TrieKeyExists(g_tRemoveEntities, szClassname)) {
		set_entvar(iEntity, var_flags, FL_KILLME);
		return FMRES_SUPERCEDE;
	}
	return FMRES_IGNORED;
}